@@ -1,25 +1,39 @@
 package org.ddahl.rscala.server
 
+import org.ddahl.rscala._
+import Protocol._
 import scala.tools.nsc.interpreter.IMain
 import scala.tools.nsc.interpreter.IR.Success
 import scala.tools.nsc.Settings
 import java.io._
-import java.nio.ByteBuffer
-
-import org.ddahl.rscala._
-import Protocol._
  
-class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, baosOut: ByteArrayOutputStream, baosErr: ByteArrayOutputStream, portFilename: String, debugger: Debugger, serializeOutput: Boolean, rowMajor: Boolean, port: Int, bufferSize: Int) {
+class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, baosOut: ByteArrayOutputStream, baosErr: ByteArrayOutputStream, snippetFilename: String, portsFilename: String, debugger: Debugger, serializeOutput: Boolean, rowMajor: Boolean, port: Int, bufferSize: Int, prioritizeConnect: Boolean) {
 
-  private val socket = new ScalaSocket(portFilename,port,bufferSize,debugger)
+  private val R = RClient(this,null,null,debugger,serializeOutput,rowMajor)
 
+  def initialInterpret() {
+    if ( repl.bind("R","org.ddahl.rscala.RClient",R) != Success ) sys.error("Problem binding R.  Interpreter is dead.")
+    val snippet = try {
+      val bufferedSource = scala.io.Source.fromFile(snippetFilename)
+      val snip = "import org.ddahl.rscala.{EphemeralReference, PersistentReference}\n" + bufferedSource.getLines.mkString("\n")
+      bufferedSource.close
+      snip
+    } catch {
+      case _: Throwable => sys.exit(0)
+    }
+    if ( repl.interpret(snippet) != Success ) sys.error("Problem interpreting initial snippet.  Interpreter is dead.")
+  }
+
+  if ( ! prioritizeConnect ) initialInterpret()
+
+  private val socket = new ScalaSocket(portsFilename,port,bufferSize,debugger)
   socket.putScalarInt(OK)
   socket.flush()
 
-  private val R = RClient(this,null,socket,debugger,serializeOutput,rowMajor)
+  if ( prioritizeConnect ) initialInterpret()
+
+  R.socket = socket
 
-  if ( repl.bind("R","org.ddahl.rscala.RClient",R) != Success ) sys.error("Problem binding R.")
-  if ( repl.interpret("import org.ddahl.rscala.{EphemeralReference, PersistentReference}") != Success ) sys.error("Problem interpreting import statement.")
   if ( serializeOutput ) {
     baosOut.reset
     baosErr.reset
@@ -28,8 +42,8 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
   private[rscala] val cacheMap = new Cache()
 
   private var functionResult: (Any, String) = null
-  private val nullary = Class.forName("scala.Function0").getMethod("apply")
-  nullary.setAccessible(true)
+  private val unary = Class.forName("scala.Function1").getMethod("apply", classOf[java.lang.Object])
+  unary.setAccessible(true)
   private val functionMap = new scala.collection.mutable.HashMap[String,(Any,String)]()
   private val functionMap2 = new scala.collection.mutable.HashMap[String,String]()
 
@@ -46,6 +60,19 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
     repl.bind(identifier,t,v)
   }
 
+  private def doAddClassPath(): Unit = {
+    try {
+      val path = new File(socket.getScalarString()).toURI.toURL
+      repl.addUrlsToClassPath(path)
+      socket.putScalarInt(OK)
+    } catch {
+      case e: Throwable =>
+        socket.putScalarInt(ERROR)
+        e.printStackTrace(pw)
+        pw.println(e + ( if ( e.getCause != null ) System.lineSeparator + e.getCause else "" ) )
+    }
+  }
+
   private def doFree(): Unit = {
     val nItems = socket.getScalarInt()
     if ( debugger.value ) debugger.msg("Freeing "+nItems+" cached items no longer needed by R interpreter.")
@@ -77,7 +104,7 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
   }
 
   private def doDef(): Unit = {
-    val body = "() => {\n" + socket.getScalarString() + "\n}"
+    val body = "($0: String) => {\n" + socket.getScalarString() + "\n}"
     try {
       if ( debugger.value ) debugger.msg("Function is: "+body)
       val (f, returnType) = if ( ! functionMap2.contains(body) ) {
@@ -89,7 +116,11 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
         }
         val functionName = repl.mostRecentVar
         val f = repl.valueOfTerm(functionName).get
-        val returnType = repl.symbolOfLine(functionName).info.toString.substring(6)  // Drop "() => " in the return type.
+        val returnType = {
+          val r = repl.symbolOfLine(functionName).info.toString.substring(10)  // Drop "String => " in the return type.
+          if ( r.startsWith("iw$") ) r.substring(3)
+          else r
+        }
         functionMap2(body) = functionName
         functionMap(functionName) = (f, returnType)
         if ( debugger.value ) debugger.msg("Function definition is okay.")
@@ -109,7 +140,7 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
     val functionName = socket.getScalarString()
     try {
       val (f, returnType) = functionMap(functionName)
-      functionResult = (nullary.invoke(f), returnType)
+      functionResult = (unary.invoke(f,socket.getScalarString()), returnType)
       R.exit()
       if ( debugger.value ) debugger.msg("Function invocation is okay.")
       socket.putScalarInt(OK)
@@ -230,7 +261,7 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
         socket.putScalarInt(UNDEFINED_IDENTIFIER)
         return
       } else {
-        if ( debugger.value ) debugger.msg("... which is empty [due to quirk in 2.10 series].")
+        if ( debugger.value ) debugger.msg("... which is empty [due to quirk that should no longer be present].")
         socket.putScalarInt(NULLTYPE)
         return
       }
@@ -398,6 +429,8 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
       case EXIT =>
         if ( debugger.value ) debugger.msg("Exiting")
         return false
+      case ADD_CLASS_PATH =>
+        doAddClassPath()
       case FREE =>
         doFree()
       case EVAL =>
@@ -448,7 +481,7 @@ class ScalaServer private (private[rscala] val repl: IMain, pw: PrintWriter, bao
 
 object ScalaServer {
 
-  def apply(portFilename: String, debug: Boolean = false, serializeOutput: Boolean = false, rowMajor: Boolean = true, port: Int = 0): ScalaServer = {
+  def apply(snippetFilename: String, portsFilename: String, debug: Boolean = false, serializeOutput: Boolean = false, rowMajor: Boolean = true, port: Int = 0, prioritizeConnect: Boolean = true): ScalaServer = {
     // Set classpath
     val settings = new Settings()
     settings.embeddedDefaults[RClient]
@@ -461,17 +494,7 @@ object ScalaServer {
     settings.deprecation.value = true
     settings.feature.value = true
     settings.unchecked.value = true
-    // Allow reflective calls without a warning since we make us of them.
-    // Use refective since Scala 2.10.x doess not have the settings.language.add method.
-    val m1 = if ( util.Properties.versionNumberString.split("""\.""").take(2).mkString(".") == "2.10" ) {
-      settings.language.getClass.getMethod("appendToValue","".getClass)
-    } else {
-      settings.language.getClass.getMethod("add","".getClass)
-    }
-    m1.setAccessible(true)
-    m1.invoke(settings.language,"reflectiveCalls")
-    // A better way to do it, but Scala 2.10.x does not the settings.language.add method.
-    // settings.language.add("reflectiveCalls")
+    settings.language.add("reflectiveCalls")
     // Set up sinks
     val (debugger,prntWrtr,baosOut,baosErr) = serializeOutput match {
       case true =>
@@ -492,12 +515,12 @@ object ScalaServer {
     val m2 = intp.getClass.getMethod("printResults_$eq",java.lang.Boolean.TYPE)
     m2.setAccessible(true)
     m2.invoke(intp,java.lang.Boolean.FALSE)
-    // Another way to do it, but Scala 2.10.x is chatty and prints "Switched off result printing."
+    // Another way to do it, but older version of Scala are chatty and print "Switched off result printing."
     // val iloop = new ILoop()
     // iloop.intp = intp
     // iloop.verbosity()
     // Return the server
-    new ScalaServer(intp, prntWrtr, baosOut, baosErr, portFilename, debugger, serializeOutput, rowMajor, port, 1*1024*1024)
+    new ScalaServer(intp, prntWrtr, baosOut, baosErr, snippetFilename, portsFilename, debugger, serializeOutput, rowMajor, port, 1*1024*1024, prioritizeConnect)
   }
 
 }
